<?php

namespace Bazookas\AdminBundle\AdminElements\Base;

use Bazookas\AdminBundle\AdminElements\Interfaces\AdminElementInterface;
use Bazookas\CommonBundle\Security\Roles;
use Symfony\Component\OptionsResolver\OptionsResolver;

class BaseAdminElement implements AdminElementInterface
{
  /**
   * @var int
   */
  protected static $globalUniqueId = 0;

  /**
   * @var int
   */
  private $uniqueId;

  /**
   * @var AdminElementInterface[]
   */
  protected $children;

  /**
   * @var array
   */
  protected $options;

  public function __construct($children = [], $options = []) {
    //set children
    if (is_array($children)) {
      $this->children = $children;
    } else {
      $this->children = [$children];
    }

    $optionsResolver = $this->getOptionsResolver();
    $this->options =  $optionsResolver->resolve($options);

    $this->uniqueId = ++self::$globalUniqueId;
  }

  /**
   * Get the default options defined for this element.
   *
   * Override this function as needed to customise your AdminElement
   * @return array
   */
  protected function getDefaultOptions() {
    return [
      'roles' => [ Roles::ROLE_CMS_USER ],
      'template' => '',
      'class' => '',
      'value' => null,
      'label' => '',
      'colClass' => '',
      'icon' => '', // for things like tabbed panel or flippable container,
      'iconClass' => '',
      'navName' => ''
    ];
  }

  /**
   * @return OptionsResolver
   */
  protected function getOptionsResolver() {
    $optionsResolver = new OptionsResolver();
    $optionsResolver
      ->setDefaults($this->getDefaultOptions())
    ;

    return $optionsResolver;
  }

  /**
   * @return string
   */
  public function getTemplate() {
    return $this->options['template'];
  }

  /**
   * @param string $template
   * @return $this
   */
  public function setTemplate($template)
  {
    $this->options['template'] = $template;
    return $this;
  }

  /**
   * @return AdminElementInterface[]
   */
  public function getChildren() {
    return $this->children;
  }

  /**
   * @param AdminElementInterface[] $children
   * @return $this
   */
  public function setChildren(array $children) {
    $this->children = $children;
    return $this;
  }

  /**
   * @param AdminElementInterface $child
   * @return $this
   */
  public function addChild(AdminElementInterface $child) {
    $this->children[] = $child;
    return $this;
  }

  /**
   * @param AdminElementInterface $child
   * @return $this
   */
  public function removeChild(AdminElementInterface $child) {
    $key = array_search($child, $this->children);
    if ($key !== false) {
      unset($this->children[$key]);
      array_splice($this->children, $key, 1);
    }

    return $this;
  }

  /**
   * @return array
   */
  public function getOptions() {
    return $this->options;
  }

  /**
   * @param array $options
   * @return $this
   */
  public function setOptions(array $options) {
    $optionsResolver = $this->getOptionsResolver();
    $this->options =  $optionsResolver->resolve($options);

    return $this;
  }

  /**
   * @return string[]
   */
  public function getRoles()
  {
    return $this->options['roles'];
  }

  /**
   * @param array $roles
   * @return $this
   */
  public function setRoles(array $roles)
  {
    $this->options['roles'] = $roles;

    return $this;
  }

  /**
   * @return string
   */
  public function getClass() {
    return $this->options['class'] . ' ' . $this->options['iconClass'];
  }

  /**
   * @return int
   */
  public function getUniqueId(){
    return $this->uniqueId;
  }

  /**
   * @return string
   */
  public function getLabel() {
    return $this->options['label'];
  }

  /**
   * @return mixed
   */
  public function getValue() {
    return $this->options['value'];
  }

  /**
   * @return string
   */
  public function getIconClass() {
    return $this->options['icon'] . ' ' . $this->options['iconClass'];
  }

  /**
   * @return string
   */
  public function getNavName() {
    return $this->options['navName'];
  }

  /**
   * @return string
   */
  public function getColClass() {
    return $this->options['colClass'];
  }
}
