<?php

namespace Bazookas\AdminBundle\PageBuilder\Base;

use Bazookas\AdminBundle\AdminElements\Containers\DivContainer;
use Bazookas\AdminBundle\AdminElements\Interfaces\AdminElementInterface;
use Bazookas\AdminBundle\AdminElements\Elements\Actions\Base\BaseActionElement;
use Bazookas\AdminBundle\AdminElements\Elements\Actions\Base\PageActionElement;
use Bazookas\AdminBundle\PageBuilder\Interfaces\PageBuilderInterface;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\HttpFoundation\Request;

abstract class BasePageBuilder implements PageBuilderInterface
{

  /**
   * @var EntityManagerInterface
   */
  protected $entityManager;

  /**
   * The entity for which the list controller applies
   * @var string
   */
  protected $entityClass;

  /**
   * @var AdminElementInterface[]
   */
  protected $elements = [];

  /**
   * @var array
   */
  protected $actions = [];

  /**
   * The short class name
   * @var string
   */
  protected $entityShortName;

  /**
   * Get the entity for which we're building a page
   * @return string
   */
  public function getEntityClass()
  {
    return $this->entityClass;
  }

  /**
   * Set the entity for which we're building a page
   * @param string $entityClass
   * @return BasePageBuilder
   */
  public function setEntityClass($entityClass)
  {
    $this->entityClass = $entityClass;

    return $this;
  }

  /**
   * @return EntityManagerInterface
   */
  public function getEntityManager()
  {
    return $this->entityManager;
  }

  /**
   * @param EntityManagerInterface $entityManager
   * @return BasePageBuilder
   */
  public function setEntityManager(EntityManagerInterface $entityManager)
  {
    $this->entityManager = $entityManager;

    return $this;
  }

  /**
   * @return AdminElementInterface[]
   */
  public function getElements()
  {
    return $this->elements;
  }

  /**
   * @param AdminElementInterface[] $elements
   * @return BasePageBuilder
   */
  public function setElements(array $elements)
  {
    $this->elements = $elements;

    return $this;
  }

  /**
   * @param AdminElementInterface $element
   * @return $this
   */
  public function addElement(AdminElementInterface $element)
  {
    $this->elements[] = $element;

    return $this;
  }

  /**
   * Get the short class name of the entity
   * @return string
   */
  public function getEntityShortClassName() {
    if (empty($this->entityShortName)) {
      $refClass = new \ReflectionClass($this->entityClass);
      $this->entityShortName = $refClass->getShortName();
    }

    return $this->entityShortName;
  }

  /**
   * Add a new action element
   * @param BaseActionElement $action
   * @return $this
   */
  public function addAction(BaseActionElement $action) {
    $this->actions[] = $action;
    return $this;
  }

  /**
   * Add a new action element to the beginning of the array
   * @param BaseActionElement $action
   * @return $this
   */
  public function prependAction(BaseActionElement $action) {
    array_unshift($this->actions, $action);
    return $this;
  }

  /**
   * @return BaseActionElement[]
   */
  public function getActions()
  {
    return $this->actions;
  }

  /**
   * @param BaseActionElement[] $actions
   * @return $this
   */
  public function setActions(array $actions)
  {
    $this->actions = $actions;

    return $this;
  }

  /**
   * Create a container containing all of the page actions
   * @return DivContainer
   */
  protected function createPageActionContainer() {
    $pageActions = [];
    foreach($this->actions as $action) {
      if ($action instanceof PageActionElement) {
        $pageActions[] = $action;
      }
    }

    return new DivContainer($pageActions, [
      'class' => 'page-actions pull-right'
    ]);
  }

  /**
   * @return array template data
   */
  public function getTemplateData()
  {
    return [];
  }

  /**
   * Builds the page
   * @param Request $request
   * @return AdminElementInterface[]
   */
  public abstract function buildPage(Request $request);

}
