## Conditional visibility

This functionality allows you to hide or show fields based on the value of another field.

### Getting started
Let's say you have a form which should only contain a question field if the type of the entity is something different from "TEXT". In this case the form would look something like this:
```php
<?php

namespace WebsiteBundle\Form\Admin;

use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\Extension\Core\Type\TextareaType;
use Symfony\Component\Form\FormBuilderInterface;
use WebsiteBundle\Entity\PsychEducationPanel;

class PsychEducationPanelAdminType extends AbstractType
{
  public function buildForm(FormBuilderInterface $builder, array $options)
  {
    $builder
      ->add('type', ChoiceType::class, [
        'label' => 'admin.entities.psychEducationPanel.fields.type',
        'choices' => [
          'admin.entities.psychEducationPanel.enums.TYPE_TEXT' => PsychEducationPanel::TYPE_TEXT,
          'admin.entities.psychEducationPanel.enums.TYPE_RADIO_QUESTION' => PsychEducationPanel::TYPE_RADIO_QUESTION,
          'admin.entities.psychEducationPanel.enums.TYPE_CHECKBOX_QUESTION' => PsychEducationPanel::TYPE_CHECKBOX_QUESTION,
        ],
        'attr' => [
          'class' => 'js-psych-education-panel-type select2 form-control'
        ]
      ])
      ->add('question', TextareaType::class, [
        'label' => 'admin.entities.psychEducationPanel.fields.question',
        'required' => false,
        'conditionallyVisible' => [
          '.js-psych-education-panel-type' => '$element.val() != "'.PsychEducationPanel::TYPE_TEXT.'"'
        ],
        'conditionalScope' => '.js-nested-form-item'
      ])
    ;
  }
}
```

The conditional visibility functionality allows you to add the following options to any form type:
- `conditionallyVisible`
  - An associative array
    - where the key is a jQuery/css selector
        - for example `#media_admin_type`
        - preferably an ID selector, faster and more precise, but any type of selector is supported
    - where the value is the condition that must be fulfilled to make item visible
        - like `"$element.val() == 2"`
        - where `$element` is automatically set to the element matched by the above selector
        - condition is evaluated by javascript `eval` (awesome, right?)
- `conditionalScope`
  - this allows you to scope your variable. This is especially useful with nested form items. In the example above the `conditionalScope` ensures that the `type` and the `question` field must both be within the same parent element `.js-nested-form-item` in order for the visibility change to be carried out. This field defaults to the `body` element.

## Conditional detachment

This functionality is very similar to conditionally visible it allows you to attach or detach fields based on the value of another field. Use this if the fields must be detached from DOM to prevent posting/validation.

So let's say you have a form which should only contain a question field if the type of the entity is something different from "TEXT". If the type of the field is "TEXT" then the question field should not be submitted. In this case the form would look something like this:

```php
<?php

namespace WebsiteBundle\Form\Admin;

use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\Extension\Core\Type\TextareaType;
use Symfony\Component\Form\FormBuilderInterface;
use WebsiteBundle\Entity\PsychEducationPanel;

class PsychEducationPanelAdminType extends AbstractType
{
  public function buildForm(FormBuilderInterface $builder, array $options)
  {
    $builder
      ->add('type', ChoiceType::class, [
        'label' => 'admin.entities.psychEducationPanel.fields.type',
        'choices' => [
          'admin.entities.psychEducationPanel.enums.TYPE_TEXT' => PsychEducationPanel::TYPE_TEXT,
          'admin.entities.psychEducationPanel.enums.TYPE_RADIO_QUESTION' => PsychEducationPanel::TYPE_RADIO_QUESTION,
          'admin.entities.psychEducationPanel.enums.TYPE_CHECKBOX_QUESTION' => PsychEducationPanel::TYPE_CHECKBOX_QUESTION,
        ],
        'attr' => [
          'class' => 'js-psych-education-panel-type select2 form-control'
        ]
      ])
      ->add('question', TextareaType::class, [
        'label' => 'admin.entities.psychEducationPanel.fields.question',
        'required' => false,
        'conditionallyDetached' => [
          '.js-psych-education-panel-type' => '$element.val() != "'.PsychEducationPanel::TYPE_TEXT.'"'
        ],
        'conditionalScope' => '.js-nested-form-item'
      ])
    ;
  }
}
```
