<?php

namespace WebsiteBundle\Controller\Admin;

use Bazookas\AdminBundle\AdminElements\Elements\Actions\ListActions\ViewRowActionElement;
use Bazookas\AdminBundle\AdminElements\Elements\Actions\PageActions\CancelPageActionElement;
use Bazookas\AdminBundle\AdminElements\Elements\Actions\PageActions\ExportPageActionElement;
use Bazookas\AdminBundle\Controller\Base\BaseAdminListController;
use Bazookas\AdminBundle\PageBuilder\Interfaces\ListPageBuilderInterface;
use Bazookas\AdminBundle\PageBuilder\ListPageBuilder;
use Bazookas\ExportBundle\Services\Interfaces\ExportServiceInterface;
use Symfony\Component\HttpFoundation\Request;
use WebsiteBundle\Entity\TestAnswer;
use WebsiteBundle\Entity\TestSession;

class TestSessionAdminController extends BaseAdminListController
{

  const ACTION_ANSWER_EXPORT = 'exportAnswer';

  public function __construct() {
    parent::__construct();

    $this->builders[self::ACTION_VIEW] = ListPageBuilder::class;
  }

  protected function getEntityClass()
  {
    return TestSession::class;
  }

  protected function modifyListBuilder(Request $request, ListPageBuilderInterface $builder)
  {
    /**
     * @var ListPageBuilder $builder
     */
    $builder = parent::modifyListBuilder($request, $builder);

    $participantId = $request->get('participantId');

    //change the route for the view action
    $actions = $builder->getActions();
    foreach($actions as $action) {
      if ($action instanceof ViewRowActionElement) {
        $action
          ->setRoute('admin_test_sessions')
          ->setRouteParams([
            'participantId' => $participantId
          ])
        ;
        break;
      }
    }

    $builder->setListQueryParams([
      'participantId' => $participantId
    ]);

    $builder
      ->addField('type')
      ->addField('number')
      ->addDateTimeField('scheduled')
      ->addDateTimeField('completed')
      ->addField('correctCount', [
        'template' => 'WebsiteBundle:List\Fields:correctCountField.html.twig'
      ])
      ->addField('avgConsecutiveCorrect')
      ->addField('avgConsecutiveWrong')
      ->addField('device')
      ->addField('inputType')
      ->addField('meanReactionTime')
      ->addField('initialInterval')
      ->addField('minInterval')
      ->addField('maxInterval')
      ->addField('medianInterval')
      ->addField('moodAnswers', [
        'template' => 'WebsiteBundle:List\Fields:moodQuestionsField.html.twig'
      ])
      ->addField('comment')
    ;

    //Add the export action
    if ($this->hasAccess(self::ACTION_EXPORT)) {
      $builder->prependAction(new ExportPageActionElement([], [
        'route' => 'admin_test_sessions',
        'action' => self::ACTION_EXPORT,
        'routeParams' => [
          'participantId' => $participantId
        ]
      ]));
    }

    $builder->prependAction(new CancelPageActionElement([], [
      'route' => 'admin_participants',
      'label' => 'admin.actions.back'
    ]));

    return $builder;
  }

  protected function getFormClass($action) {
    return null;
  }

  protected function hasAccess($action) {
    switch($action) {
      case self::ACTION_LIST:
      case self::ACTION_VIEW:
      case self::ACTION_EXPORT:
      case self::ACTION_ANSWER_EXPORT:
        return true;
      default:
        return false;
    }
  }

  protected function modifyViewBuilder(Request $request, ListPageBuilder $builder) {
    $builder
      ->setListQueryParams([
        'sessionId' => $request->get('_id')
      ])

      ->setEntityClass(TestAnswer::class)
      ->setLimit(100)
      ->setPageAction(self::ACTION_VIEW)

      ->addField('trial')
      ->addField('stimulus')
      ->addField('answer')
      ->addField('correctAnswer')
      ->addBooleanField('correct')
      ->addField('reactionTime')
      ->addField('interval')
      ->addField('consecutiveCorrect')
      ->addField('consecutiveWrong')
      ->addField('level')
      ->addField('score')
    ;

    //Add the export action
    if ($this->hasAccess(self::ACTION_EXPORT)) {
      $builder->prependAction(new ExportPageActionElement([], [
        'route' => 'admin_test_sessions',
        'action' => self::ACTION_ANSWER_EXPORT,
        'routeParams' => [
          'sessionId' => $request->get('_id')
        ]
      ]));
    }

    $builder->prependAction(new CancelPageActionElement([], [
      'route' => 'admin_participants',
      'label' => 'admin.actions.back',
      'routeParams' => [
        '_id' => $request->get('participantId'),
        '_action' => self::ACTION_VIEW
      ]
    ]));

    return $builder;
  }

  /**
   * Export the session totals
   * @param Request $request
   * @param $action
   * @return \Symfony\Component\HttpFoundation\Response
   * @throws \Exception
   */
  protected function exportAction(Request $request, $action) {
    $participantId = $request->get('participantId');

    $entityManager = $this->get('doctrine.orm.entity_manager');
    $testSessionRepo = $entityManager->getRepository(TestSession::class);

    $config = [
      'sheets' => [
        [
          'entity' => TestSession::class,
          'fields' => [
            'participant.id' => 'admin.entities.participant.fields.id',
            'number' => 'admin.entities.testSession.fields.number',
            'scheduled'  => 'admin.entities.testSession.fields.scheduled',
            'completed'  => 'admin.entities.testSession.fields.completed',
            'duration'  => 'admin.entities.testSession.fields.duration',
            'trialCount'  => 'admin.entities.testSession.fields.trialCount',
            'correctCount'  => 'admin.entities.testSession.fields.correctCount',
            'avgConsecutiveCorrect'  => 'admin.entities.testSession.fields.avgConsecutiveCorrect',
            'avgConsecutiveWrong'  => 'admin.entities.testSession.fields.avgConsecutiveWrong',
            'device'  => 'admin.entities.testSession.fields.device',
            'inputType'  => 'admin.entities.testSession.fields.inputType',
            'meanReactionTime'  => 'admin.entities.testSession.fields.meanReactionTime',
            'initialInterval'  => 'admin.entities.testSession.fields.initialInterval',
            'minInterval'  => 'admin.entities.testSession.fields.minInterval',
            'maxInterval'  => 'admin.entities.testSession.fields.maxInterval',
            'medianInterval'  => 'admin.entities.testSession.fields.medianInterval',
            'totalScore' => 'admin.entities.testSession.fields.totalScore',
            'finalLevel' => 'admin.entities.testSession.fields.finalLevel',
            'type'  => 'admin.entities.testSession.fields.type',
            'gamificationLevel' => 'admin.entities.testSession.fields.gamificationLevel',
            'comment'  => 'admin.entities.testSession.fields.comment'
          ],
          'queryBuilder' => $testSessionRepo->createParticipantExportQueryBuilder($participantId)
        ]
      ],
      'fileName' => 'pp'.$participantId.'_sessions_' . date('Y-m-d'),
      'dataService' => 'WebsiteBundle.Services.TestAnswerExportDataService'
    ];

    $exportService = $this->get('bazookas.export.service.ExportService');

    if (!$exportService instanceof ExportServiceInterface) {
      throw new \Exception('Invalid export service, must implement ' . ExportServiceInterface::class);
    }

    return $exportService->export($request, $config);
  }

  /**
   * Export individual answers per session
   * @param Request $request
   * @param $action
   * @return \Symfony\Component\HttpFoundation\Response
   * @throws \Exception
   */
  protected function exportAnswerAction(Request $request, $action) {
    $sessionId = $request->query->get('sessionId');

    $entityManager = $this->get('doctrine.orm.entity_manager');

    $sessionRepo = $entityManager->getRepository(TestSession::class);
    $participantId = $sessionRepo->getParticipantId($sessionId);
    $session = $sessionRepo->findOneById($sessionId);

    $answerRepo = $entityManager->getRepository(TestAnswer::class);

    $config = [
      'sheets' => [
        [
          'entity' => TestAnswer::class,
          'fields' => [
            'participant.id' => 'admin.entities.participant.fields.id',
            'session.number' => 'admin.entities.testSession.fields.number',
            'trial' => 'admin.entities.testAnswer.fields.trial',
            'stimulus' => 'admin.entities.testAnswer.fields.stimulus',
            'answer' => 'admin.entities.testAnswer.fields.answer',
            'correctAnswer' => 'admin.entities.testAnswer.fields.correctAnswer',
            'correct' => 'admin.entities.testAnswer.fields.correct',
            'reactionTime' => 'admin.entities.testAnswer.fields.reactionTime',
            'interval' => 'admin.entities.testAnswer.fields.interval',
            'consecutiveCorrect' => 'admin.entities.testAnswer.fields.consecutiveCorrect',
            'consecutiveWrong' => 'admin.entities.testAnswer.fields.consecutiveWrong',
            'level' => 'admin.entities.testAnswer.fields.level',
            'score' => 'admin.entities.testAnswer.fields.score',
          ],
          'queryBuilder' => $answerRepo->createAnswerExportQuery($sessionId)
        ]
      ],
      'fileName' => 'pp'.$participantId.'_session'.$session->getNumber().'_' . date('Y-m-d'),
    ];

    $exportService = $this->get('bazookas.export.service.ExportService');

    if (!$exportService instanceof ExportServiceInterface) {
      throw new \Exception('Invalid export service, must implement ' . ExportServiceInterface::class);
    }

    return $exportService->export($request, $config);
  }
}
