<?php

namespace Bazookas\CommonBundle\Command\Deploy;

use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;
use Symfony\Component\Console\Input\ArrayInput;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Question\ConfirmationQuestion;
use Symfony\Component\Console\Question\Question;
use Symfony\Component\Process\Process;
use Symfony\Component\Filesystem\Filesystem;
use Symfony\Component\Yaml\Yaml;

class BuildCommand extends ContainerAwareCommand
{
  private $output;
  private $command = 'project:deploy:build';
  private $kernelRoot;

  protected function configure()
  {
    $this
      ->setName($this->command)
      ->setDescription("Creates a tar of the entire project, excluding some files that may not be overwritten on the server.")
    ;
  }

  protected function execute(InputInterface $input, OutputInterface $output)
  {
    $this->output = $output;
    $this->kernelRoot = $this->getContainer()->get('kernel')->getRootDir();
    // No need for a command lock since this will be run manually

    $output->writeln("<comment>".date('Y-m-d H:i:s')." - Starting build...</comment>");

    // ask the user for a new asset version
    $this->updateAssetVersion($input, $output);

    //Optimize the composer autoloader
    $this->optimizeAutoloader($output);

    // install static assets (like images and such)
    $this->installAssets($output);

    // build gulp
    $this->gulpBuild($output);

    // create tar package
    $this->createPackage($output);

    $output->writeln("<info>".date('Y-m-d H:i:s')." - Done</info>");
  }

  private function optimizeAutoloader(OutputInterface $output) {
    $output->writeln("<comment>".date('Y-m-d H:i:s')." - Running composer optimize autoloader...</comment>");

    $appRootDir = $this->getContainer()->getParameter('kernel.root_dir');
    $fs = new Filesystem();
    if ($fs->exists($appRootDir.'/../composer.phar')) {
      $command = 'composer.phar dump-autoload --optimize --no-dev --classmap-authoritative';
    } else {
      $command = 'composer dump-autoload --optimize --no-dev --classmap-authoritative';
    }

    $this->attemptCommand($output, $command);
  }

  private function installAssets($output) {
    $command = $this->getApplication()->find('assets:install');

    $arguments = array(
      'command' => 'assets:install',
      '--symlink' => true,
      '--relative' => true,
    );

    $commandInput = new ArrayInput($arguments);
    $returnCode = $command->run($commandInput, $output);
    // do something with return code?
  }

  private function updateAssetVersion(InputInterface $input, OutputInterface $output) {
    $oldYml = file_get_contents($this->kernelRoot .'/config/config.yml');
    $yml = Yaml::parse($oldYml);
    $version = strval($yml['framework']['assets']['version']);
    $output->writeln("Please provide a new asset version number, leave empty to ignore <info>(currently $version)</info>");

    $questionHelper = $this->getHelper('question');

    $question = new Question('framework.assets.version: ');
    $answer = $questionHelper->ask($input, $output, $question);

    $continue = false;
    if (!empty($answer) && $answer != $version) {
      $yml['framework']['assets']['version'] = $answer;

      $pregVersion = preg_quote($version);
      $regex = '/version: (\''.$pregVersion.'\'|"'.$pregVersion.'"|'.$pregVersion.')/';

      // TODO better yml interpretation
      if (preg_match_all($regex, $oldYml) === 1) {
        $newYml = preg_replace($regex, "version: $answer", $oldYml);
        $continue = true;
      } else {
        $confirmation = new ConfirmationQuestion('Can not update asset version without changing file structure, do you want to continue (y/<info>n</info>)?', false);
        $continue = $questionHelper->ask($input, $output, $confirmation);
        $newYml = Yaml::dump($yml, 5, 2);
      }
    }

    if ($continue && isset($newYml)) {
      file_put_contents($this->kernelRoot .'/config/config.yml', $newYml);
      $output->writeln("<comment>Changed version number to </comment> <info>$answer</info>");
    }
    else {
      $output->writeln("<comment>Version number left unchanged</comment>");
    }
  }

  private function gulpBuild(OutputInterface $output) {
    $this->attemptCommand($output, 'gulp clean && NODE_ENV=production gulp build');
  }

  private function createPackage(OutputInterface $output) {
    $output->writeln("<comment>".date('Y-m-d H:i:s')." - Tarring files...</comment>");

    // make sure the build folder exists
    $filesystem = new Filesystem();
    $filesystem->mkdir($this->kernelRoot . '/../build');
    dump($this->kernelRoot . '/build');

    // define options here to keep everything readable
    $options = [
      '--exclude="./app/config/parameters.yml"',
      '--exclude="./web/uploads/"', // uploads folder
      '--exclude="./web/media/"', // imagine media cache
      '--exclude="./app/Resources/translations"', // imagine media cache
    ];

    // define the command and insert optiopns
    // Important note: options MUST BE defined before the -f flag
    // http://superuser.com/a/559341/582064
    $now = new \DateTime();
    $command = 'tar -zc %options% -f ./build/build-'.$now->format('Y-m-d').'.tar.gz app src vendor web .htaccess .user.ini bin';
    $command = str_replace('%options%', implode(' ', $options), $command);

    if ($this->attemptCommand($output, $command, false, false)) {
      $output->writeln('<info>build/build-'.$now->format('Y-m-d').'.tar.gz file created, upload file to project root and use <comment>tar -zxf</comment> to extract on server. (optionally add <comment>-v</comment> for verbose output)</info>');
    }
  }

  private function attemptCommand(OutputInterface $output, $command, $showOutput = true, $showErrorOutput = true) {
    $output->writeln("<comment>".date('Y-m-d H:i:s')." - Attempting command '$command'</comment>");

    $process = new Process($command);
    $process->run();

    // executes after the command finishes
    if ($showOutput) {
      $output->writeln("<info>".$process->getOutput()."</info>");
    }

    if ($process->isSuccessful()) {
      $output->writeln("<comment>".date('Y-m-d H:i:s')." - Successfully executed '$command'</comment>");
      return true;
    }
    else {
      if ($showErrorOutput) {
        $output->writeln("<error>".$process->getErrorOutput()."</error>");
      }
      $output->writeln("<error>".date('Y-m-d H:i:s')." - Command failed '$command'</error>");

      return false;
    }
  }

}
