<?php

namespace Bazookas\ExportBundle\Services\Import\Base;

use Bazookas\CommonBundle\Utility\Utility;
use Bazookas\ExportBundle\Entity\GenericFileEntity;
use Bazookas\ExportBundle\Exception\ImportException;
use Doctrine\ORM\EntityManager;


abstract class BaseImportService
{
  /**
   * @var EntityManager
   */
  protected $entityManager;

  /**
   * @var string
   */
  protected $importDirectory;

  /**
   * BaseImportService constructor.
   * @param EntityManager $entityManager
   * @param string $projectRootDirectory
   */
  public function __construct(EntityManager $entityManager, string $projectRootDirectory) {
    $this->entityManager = $entityManager;
    $this->importDirectory = $projectRootDirectory . '/web/uploads/imports';
  }

  /**
   * @param GenericFileEntity $entity
   * @return GenericFileEntity
   */
  public function handleUpload(GenericFileEntity $entity) {
    $file = $entity->getFile();

    $fileName = md5(uniqid() . '.' . $file->guessExtension());
    $entity
      ->setOriginalFileName($file->getClientOriginalName() .'.'.$file->guessExtension())
      ->setFileName($fileName)
      ->setPath($this->importDirectory . '/'.$fileName)
    ;

    $file->move($this->importDirectory, $fileName);

    return $entity;
  }

  /**
   * @param GenericFileEntity $entity
   * @param bool $firstRowContainsHeader
   * @param bool $deleteFileAfterParse
   */
  public function process(GenericFileEntity $entity, $firstRowContainsHeader = true, $deleteFileAfterParse = true) {
    if (($handle = fopen($entity->getPath(), "r")) !== false) {
      $columnHeaders = [];

      while (($data = fgetcsv($handle, 0, $entity->getDelimiter(), $entity->getEnclosure())) !== false) {
        if ($firstRowContainsHeader) {
          if (empty($columnHeaders)) {
            $columnHeaders = $this->validateColumnHeaders($data);
            continue;
          }

          //Convert the row data to an associative array
          $data = array_combine($columnHeaders, $data);
        }

        //Skip all rows that contain only empty strings
        if (Utility::arrayContainsOnlyEmptyValues($data)) {
          continue;
        }

        $parsedData[] = $this->parseRow($data);
      }
      fclose($handle);

      //Execute a success callback
      $this->onComplete();
    } else {
      throw new ImportException('The file could not be found.');
    }

    //Should the file be deleted after it was parsed?
    if ($deleteFileAfterParse) {
      unlink($entity->getPath());
    }

  }

  /**
   * @param array $rowData
   * @return array
   */
  protected function validateColumnHeaders(array $rowData) {
    $expectedHeaders = $this->getExpectedColumnHeaders();

    // first check if it is an associative array before checking the difference
    if (Utility::isAssociative($expectedHeaders)) {
      $diff = array_diff($rowData, array_keys($expectedHeaders));
    }
    else {
      $diff = array_diff($rowData, $expectedHeaders);
    }

    if (!empty($diff)) {
      throw new ImportException('The file did not contain valid column headers. The following headers are invalid: '.print_r($diff, true).' Expected headers in order: ' . print_r(array_keys($expectedHeaders), true));
    }

    return array_values($expectedHeaders);
  }

  /**
   * @param array $rowData
   * @return array
   */
  protected abstract function parseRow(array &$rowData);

  /**
   * Either return a numerically indexed array containing the expected column headers, or an associative array
   * mapping the headers in the file to the header that should be used when creating the $rowdData arrays
   * @return array
   */
  protected abstract function getExpectedColumnHeaders();

  /**
   * This function is called when the entire document has been parsed.
   */
  protected abstract function onComplete();
}
