<?php

namespace Bazookas\MediaBundle\Services\Vimeo;

use Bazookas\MediaBundle\Entity\Interfaces\MediaEntityInterface;
use Bazookas\MediaBundle\Entity\Video;
use Bazookas\MediaBundle\Exception\MediaServiceException;
use Bazookas\MediaBundle\Services\Media\Video\Interfaces\VideoServiceInterface;
use Doctrine\ORM\EntityManagerInterface;
use Bazookas\MediaBundle\Services\Media\Interfaces\MediaServiceInterface;
use \Vimeo\Vimeo;
use Bazookas\MediaBundle\Entity\VideoProvider;
use Doctrine\ORM\Event\LifecycleEventArgs;
use Doctrine\ORM\Event\PreUpdateEventArgs;

//TODO refactor: maybe use a repository here?

class VimeoService implements VideoServiceInterface, MediaServiceInterface {

  //Quality selection strategy
  const STRATEGY_BEST_FIT = 1;
  const STRATEGY_EXACT = 2;

  protected $vimeoClient;
  private $entityManager;

  public function __construct($providers) {
    if (empty($providers)) {
      return;
    }

    $clientId = $providers['vimeo']['clientId'];
    $clientSecret = $providers['vimeo']['clientSecret'];
    $this->vimeoClient = new Vimeo($clientId, $clientSecret);
  }

  public function getProvider() {
    return Video::PROVIDER_VIMEO;
  }

  public function setEntityManager(EntityManagerInterface $entityManager) {
    $this->entityManager = $entityManager;
  }

  private function getAccessToken() {
    $provider = $this->getVideoProvider();
    return $provider->getAccessToken();
  }

  private function getVideoProvider() {
    $qb = $this->entityManager->createQueryBuilder();
    $qb
      ->select('vp')
      ->from(VideoProvider::class, 'vp')
      ->andWhere($qb->expr()->eq('vp.provider', ':provider'))
      ->setParameter('provider', Video::PROVIDER_VIMEO)
    ;

    return $qb->getQuery()->getOneOrNullResult();
  }

  public function handleUpload(LifecycleEventArgs $args, $file, MediaEntityInterface $mediaItem, $directory = null) {
    //check if the providerId field has been changed to avoid having to make unnecessary API calls
    if ($args instanceof PreUpdateEventArgs && !$args->hasChangedField('providerId')) {
      return;
    }

    $token = $this->getAccessToken();
    $this->vimeoClient->setToken($token);

    //TODO add error handling
    //TODO add rate limit error handling
    $response = $this->vimeoClient->request('/videos/'.$mediaItem->getProviderId(), [], 'GET');

    if ($response['status'] >= 200 && $response['status'] < 300 ) {
      $formats = !empty($response['body']['files']) ? $response['body']['files'] : [];

      $this->getMediaFormats($formats, $mediaItem);
      $mediaItem->setDuration($response['body']['duration']);
      $mediaItem->setThumbnail($this->getThumbnail($response['body']['pictures']['sizes']));
    } else {
      throw new MediaServiceException('media.exceptions.MediaServiceException.vimeo.apiException');
    }
  }

  private function getMediaFormats(array $formats, MediaEntityInterface $mediaItem, $formatStrategy = self::STRATEGY_BEST_FIT) {
    switch($formatStrategy) {
      case self::STRATEGY_BEST_FIT:
        return $this->getMediaFormatsByBestFit($formats, $mediaItem);
      default:
        throw new \Exception('Media format strategy not implemented.');
    }
  }

  private function getMediaFormatsByBestFit(array $availableFormats, MediaEntityInterface $mediaItem) {
    $mediaUrls = [];
    foreach(Video::getResolutionFormats() as $formatKey => $format) {
      $formatHeight = $format['h'];
      $formatFps = $format['fps'];

      $bestFitDifference = 1000;
      $bestFitFpsDifference = 1000;

      foreach($availableFormats as $availableFormat) {
        if (!isset($availableFormat['height']) && $availableFormat['fps']) {
          continue;
        }

        //difference = absolute value ( ideal height - available format height)
        $heightDifference = abs($formatHeight - $availableFormat['height']);
        $fpsDifference = abs($formatFps - $availableFormat['fps']);

        //give priority to getting the resolution right
        if ( $heightDifference < $bestFitDifference || ($heightDifference === $bestFitDifference && $fpsDifference < $bestFitFpsDifference )) {
          $bestFitDifference = $heightDifference;
          $bestFitFpsDifference = $fpsDifference;

          $link = explode('&', $availableFormat['link']);
          unset($link[count($link)-1]);
          $link = implode('&', $link);

          $mediaUrls[$formatKey] = array(
            'url' => $link,
            'aspectRatio' => $availableFormat['width'].':'.$availableFormat['height']
          );
        }
      }
    }

    $mediaItem->setResolutions($mediaUrls);
    $mediaItem->setUrl($mediaUrls['QUALITY_STANDARD']['url']);
  }

  //TODO implement this properly, dirty, dirty, hardcoding!
  private function getThumbnail(array $availableFormats) {
    //TODO normalize this by using a best fit strategy as used with the video resolutions
    foreach($availableFormats as $format) {
      if ($format['width'] === 200) {
        return $format['link'];
      }
    }
  }

}
